@@ -8,7 +8,10 @@ import sys
 import time
 import itertools
 import scipy.stats
+import gc
+import sklearn
 from joblib import Parallel, delayed
+from tqdm import tqdm
 
 
 def print_memory_use():
@@ -98,37 +101,103 @@ class AbstractAffinityMatrixFromOneD(object):
         raise NotImplementedError()
 
 
+def magnitude_norm_sparsemat(sparse_mat):
+    return sklearn.preprocessing.normalize(sparse_mat, norm='l2', axis=1)
+    #return sparse_mat.divide(sparse_mat.multiply(sparse_mat).sum(axis=-1))
+
+
+def sparse_cosine_similarity(sparse_mat_1, sparse_mat_2):
+    normed_sparse_mat_1 = magnitude_norm_sparsemat(sparse_mat=sparse_mat_1)
+    normed_sparse_mat_2 = magnitude_norm_sparsemat(sparse_mat=sparse_mat_2)
+    return normed_sparse_mat_1.dot(normed_sparse_mat_2.transpose())
+
+
+class SparseNumpyCosineSimFromFwdAndRevOneDVecs():
+
+    def __init__(self, n_neighbors, verbose, nn_n_jobs,
+                       memory_cap_gb=1.0):
+        self.n_neighbors = n_neighbors   
+        self.nn_n_jobs = nn_n_jobs
+        self.verbose = verbose
+        self.memory_cap_gb = memory_cap_gb
+
+    def __call__(self, fwd_vecs, rev_vecs):
+
+        #normalize the vectors 
+        fwd_vecs = magnitude_norm_sparsemat(sparse_mat=fwd_vecs)
+        rev_vecs = magnitude_norm_sparsemat(sparse_mat=rev_vecs)
+
+        #fwd_sims = fwd_vecs.dot(fwd_vecs.transpose())
+        #rev_sims = fwd_vecs.dot(rev_vecs.transpose())
+
+        #assuming float64 for the affinity matrix, figure out the batch size
+        # to use given the memory cap
+        memory_cap_gb = (self.memory_cap_gb if rev_vecs
+                         is None else self.memory_cap_gb/2.0)
+        batch_size = int(memory_cap_gb*(2**30)/(len(fwd_vecs)*8))
+        batch_size = min(max(1,batch_size),len(fwd_vecs))
+        if (self.verbose):
+            print("Batching in slices of size",batch_size)
+            sys.stdout.flush()
+
+        topk_cosine_sim_results = []
+        for i in tqdm(range(0,len(fwd_vecs),batch_size)):
+            topk_cosine_sim_results.append(
+                top_k_fwdandrev_dot_prod(fwd_vecs[i:i+batch_size],
+                                         rev_vecs,
+                                         fwd_vecs, self.n_neighbors+1))
+        neighbors = np.concatenate(
+                     [x[0] for x in topk_cosine_sim_results], axis=0)
+        sims = np.concatenate([x[1] for x in topk_cosine_sim_results], axis=0) 
+
+        return sims, neighbors
+
+
 class NumpyCosineSimilarity(AbstractAffinityMatrixFromOneD):
 
-    def __init__(self, verbose, gpu_batch_size=None):
+    def __init__(self, verbose, rows_per_batch=500):
         self.verbose = verbose
-        self.gpu_batch_size = gpu_batch_size
+        self.rows_per_batch = rows_per_batch
 
     def __call__(self, vecs1, vecs2):
 
         start_time = time.time()
-        normed_vecs1 = np.nan_to_num(
-                        vecs1/np.linalg.norm(vecs1, axis=1)[:,None],
-                        copy=False)
-        normed_vecs2 = np.nan_to_num(
-                        vecs2/np.linalg.norm(vecs2, axis=1)[:,None],
-                        copy=False)
-        if (self.verbose):
-            print("Normalization computed in",
-                  round(time.time()-start_time,2),"s")
-            sys.stdout.flush()
-        if (self.gpu_batch_size is not None):
-            to_return = B.matrix_dot_product(normed_vecs1, normed_vecs2.T,
-                                             batch_size=self.gpu_batch_size)
+        if (scipy.sparse.issparse(vecs1)):
+            vecs1 = magnitude_norm_sparsemat(sparse_mat=vecs1)
+            vecs2 = magnitude_norm_sparsemat(sparse_mat=vecs2)
+
+            if (self.verbose):
+                print("Batching in slices of size",self.rows_per_batch)
+                sys.stdout.flush()
+
+            transpose_vecs2 = vecs2.transpose()
+            to_return = np.zeros((vecs1.shape[0], vecs2.shape[0]))
+            for i in tqdm(range(0, vecs1.shape[0], self.rows_per_batch)):
+                to_return[i:min(i+self.rows_per_batch, vecs1.shape[0])] =\
+                    np.array(vecs1[i:i+self.rows_per_batch]
+                              .dot(transpose_vecs2).todense())
+            #to_return = vecs1.dot(vecs2.transpose())
+            #cast to dense for now
+            #to_return = np.array(to_return.todense())
         else:
+            normed_vecs1 = np.nan_to_num(
+                            vecs1/np.linalg.norm(vecs1, axis=1)[:,None],
+                            copy=False)
+            normed_vecs2 = np.nan_to_num(
+                            vecs2/np.linalg.norm(vecs2, axis=1)[:,None],
+                            copy=False)
+            if (self.verbose):
+                print("Normalization computed in",
+                      round(time.time()-start_time,2),"s")
+                sys.stdout.flush()
             #do the multiplication on the CPU
             to_return = np.dot(normed_vecs1,normed_vecs2.T)
-        end_time = time.time()
-    
-        if (self.verbose):
-            print("Cosine similarity mat computed in",
-                  round(end_time-start_time,2),"s")
-            sys.stdout.flush()
+            end_time = time.time()
+        
+            if (self.verbose):
+                print("Cosine similarity mat computed in",
+                      round(end_time-start_time,2),"s")
+                sys.stdout.flush()
 
         return to_return
 
@@ -312,7 +381,8 @@ class AffmatFromSeqletsWithNNpairs(object):
         self.pattern_comparison_settings = pattern_comparison_settings 
         self.sim_metric_on_nn_pairs = sim_metric_on_nn_pairs
 
-    def __call__(self, seqlets, filter_seqlets=None, seqlet_neighbors=None):
+    def __call__(self, seqlets, filter_seqlets=None,
+                       seqlet_neighbors=None, return_sparse=False):
         (all_fwd_data, all_rev_data) =\
             modiscocore.get_2d_data_from_patterns(
                 patterns=seqlets,
@@ -338,19 +408,57 @@ class AffmatFromSeqletsWithNNpairs(object):
                      neighbors_of_things_to_scan=seqlet_neighbors,
                      filters=filters_all_fwd_data,
                      things_to_scan=all_fwd_data,
-                     min_overlap=self.pattern_comparison_settings.min_overlap) 
+                     min_overlap=self.pattern_comparison_settings.min_overlap,
+                     return_sparse=return_sparse) 
         if (filters_all_rev_data is not None):
             affmat_rev = self.sim_metric_on_nn_pairs(
-                     neighbors_of_things_to_scan=seqlet_neighbors,
-                     filters=filters_all_rev_data,
-                     things_to_scan=all_fwd_data,
-                     min_overlap=self.pattern_comparison_settings.min_overlap) 
+                 neighbors_of_things_to_scan=seqlet_neighbors,
+                 filters=filters_all_rev_data,
+                 things_to_scan=all_fwd_data,
+                 min_overlap=self.pattern_comparison_settings.min_overlap,
+                 return_sparse=return_sparse) 
         else:
             affmat_rev = None
-        affmat = (np.maximum(affmat_fwd, affmat_rev) if
-                  (affmat_rev is not None) else np.array(affmat_fwd))
-        return affmat
-        
+        if (return_sparse==False):
+            if (len(affmat_fwd.shape)==3):
+                #dims are N x N x 2, where first entry of last idx is sim,
+                # and the second entry is the alignment.
+                if (affmat_rev is None):
+                    affmat = affmat_fwd
+                else:
+                    #will return something that's N x N x 3, where the third
+                    # entry in last dim is is_fwd 
+                    is_fwd = (affmat_fwd[:,:,0] > affmat_rev[:,:,0])*1.0
+                    affmat = np.zeros((affmat_fwd.shape[0],
+                                       affmat_fwd.shape[1],3))
+                    affmat[:,:,0:2] = (affmat_fwd*is_fwd[:,:,None]
+                                       + affmat_rev*(1-is_fwd[:,:,None]))
+                    affmat[:,:,2] = is_fwd 
+
+            else:
+                affmat = (np.maximum(affmat_fwd, affmat_rev) if
+                          (affmat_rev is not None) else np.array(affmat_fwd))
+        else:
+            if (len(affmat_fwd[0].shape)==2):
+                #dims are N x N x 2, where first entry of last idx is sim,
+                # and the second entry is the alignment.
+                if (affmat_rev is None):
+                    affmat = affmat_fwd
+                else:
+                    affmat = [] 
+                    for fwd, rev in zip(affmat_fwd, affmat_rev):
+                        is_fwd = (fwd[:,0] > rev[:,0])*1.0 
+                        new_row = np.zeros((fwd.shape[0],3))
+                        new_row[:,0:2] = (fwd*is_fwd[:,None] +
+                                          rev*(1-is_fwd[:,None]))
+                        new_row[:,2] = is_fwd 
+                        affmat.append(new_row)
+            else:
+                affmat = ([np.maximum(x,y) for (x,y)
+                           in zip(affmat_fwd, affmat_rev)]
+                          if affmat_rev is not None else affmat_fwd)
+                 
+        return affmat  
 
 
 class AbstractSimMetricOnNNpairs(object):
@@ -370,8 +478,10 @@ class ParallelCpuCrossMetricOnNNpairs(AbstractSimMetricOnNNpairs):
         self.cross_metric_single_region = cross_metric_single_region
         self.verbose = verbose
 
+    #min_overlap is w.r.t. the length of 'filters'
     def __call__(self, filters, things_to_scan, min_overlap,
-                       neighbors_of_things_to_scan=None):
+                       neighbors_of_things_to_scan=None,
+                       return_sparse=False):
         if (neighbors_of_things_to_scan is None):
             neighbors_of_things_to_scan = [list(range(len(filters)))
                                            for x in things_to_scan] 
@@ -393,24 +503,19 @@ class ParallelCpuCrossMetricOnNNpairs(AbstractSimMetricOnNNpairs):
         # only returns the best similarity and not the alignment that
         # gives rise to that similarity 
         if (self.cross_metric_single_region.returns_pos==False):
-            to_return = np.zeros((things_to_scan.shape[0], filters.shape[0]))
+            if (return_sparse==False):
+                to_return = np.zeros((things_to_scan.shape[0],
+                                      filters.shape[0]))
         else:
-            #each return value will contain both the position of the alignment
-            # as well as the similarity at that position; hence the
-            # length of the third dimension is 2.
-            # The similarity comes first, then the position
-            to_return = np.zeros((things_to_scan.shape[0],
-                                  filters.shape[0], 2))
+            if (return_sparse==False):
+                #each return value will contain both the
+                # position of the alignment
+                # as well as the similarity at that position; hence the
+                # length of the third dimension is 2.
+                # The similarity comes first, then the position
+                to_return = np.zeros((things_to_scan.shape[0],
+                                      filters.shape[0], 2))
 
-        #job_arguments = []
-
-        #for neighbors_of_thing_to_scan, thing_to_scan\
-        #    in zip(neighbors_of_things_to_scan, things_to_scan): 
-        #    args = (filters[neighbors_of_thing_to_scan], thing_to_scan) 
-        #    job_arguments.append(args)
-
-        #print("cp5"); print_memory_use();        
-         
         start = time.time()
         if (self.verbose):
             print("Launching nearest neighbors affmat calculation job")
@@ -423,22 +528,44 @@ class ParallelCpuCrossMetricOnNNpairs(AbstractSimMetricOnNNpairs):
                         things_to_scan[i])
                         for i in range(len(things_to_scan))))
 
+        assert len(results)==len(neighbors_of_things_to_scan)
+        if (self.cross_metric_single_region.returns_pos==False):
+            assert all([len(x)==len(y) for x,y in
+                        zip(results, neighbors_of_things_to_scan)])
+        else:
+            assert all([len(x[0])==len(y) for x,y in
+                        zip(results, neighbors_of_things_to_scan)])
+
         if (self.verbose):
             print("Parallel runs completed")
             print_memory_use()
             sys.stdout.flush()
 
-        for (thing_to_scan_idx, (result, thing_to_scan_neighbor_indices))\
-             in enumerate(zip(results, neighbors_of_things_to_scan)):
-            #adjust the "position" to remove the effect of the padding
-            if (self.cross_metric_single_region.returns_pos==True):
-                result[1] -= padding_amount 
-                to_return[thing_to_scan_idx,
-                          thing_to_scan_neighbor_indices] =\
-                    np.transpose(result,(1,0))
+        if (return_sparse==True):
+            to_return = []
+            if (self.cross_metric_single_region.returns_pos==False):
+                assert len(results[0].shape)==1  
+                to_return = results
             else:
-                to_return[thing_to_scan_idx,
-                          thing_to_scan_neighbor_indices] = result
+                assert len(results[0].shape)==2 
+                for result in results:
+                    #adjust the "position" to remove the effect of the padding
+                    result[1] -= padding_amount 
+                    to_return.append(np.transpose(result, (1,0)))
+        else: 
+            for (thing_to_scan_idx, (result, thing_to_scan_neighbor_indices))\
+                 in enumerate(zip(results, neighbors_of_things_to_scan)):
+                #adjust the "position" to remove the effect of the padding
+                if (self.cross_metric_single_region.returns_pos==True):
+                    result[1] -= padding_amount 
+                    to_return[thing_to_scan_idx,
+                              thing_to_scan_neighbor_indices] =\
+                        np.transpose(result,(1,0))
+                else:
+                    to_return[thing_to_scan_idx,
+                              thing_to_scan_neighbor_indices] = result
+
+        gc.collect()
 
         end = time.time()
         if (self.verbose):
