@@ -41,21 +41,19 @@ class TrimToFracSupport(AbstractAggSeqletPostprocessor):
                   verbose=self.verbose) for x in aggregated_seqlets]
 
 
-class TrimToBestWindow(AbstractAggSeqletPostprocessor):
+class AbstractTrimToBestWindow(AbstractAggSeqletPostprocessor):
 
-    def __init__(self, window_size, track_names):
+    def __init__(self, window_size):
         self.window_size = window_size
-        self.track_names = track_names
+
+    def score_positions(self, aggregated_seqlet):
+        raise NotImplementError()
 
     def __call__(self, aggregated_seqlets):
         trimmed_agg_seqlets = []
         for aggregated_seqlet in aggregated_seqlets:
             start_idx = np.argmax(util.cpu_sliding_window_sum(
-                arr=np.sum(np.abs(
-                    np.concatenate(
-                    [aggregated_seqlet[track_name].fwd
-                      .reshape(len(aggregated_seqlet),-1) for
-                     track_name in self.track_names], axis=1)),axis=1),
+                arr=self.score_positions(aggregated_seqlet),
                 window_size=self.window_size))
             end_idx = start_idx + self.window_size
             trimmed_agg_seqlets.append(
@@ -64,6 +62,21 @@ class TrimToBestWindow(AbstractAggSeqletPostprocessor):
         return trimmed_agg_seqlets
 
 
+class TrimToBestWindowByIC(AbstractTrimToBestWindow):
+
+    def __init__(self, window_size, onehot_track_name, bg_freq):
+        super(TrimToBestWindowByIC, self).__init__(window_size=window_size)
+        self.onehot_track_name = onehot_track_name
+        self.bg_freq = bg_freq
+
+    #sub up imp for each track, take l1 norm, average across seqlets
+    def score_positions(self, aggregated_seqlet):
+        ppm = aggregated_seqlet[self.onehot_track_name].fwd
+        per_pos_ic = util.compute_per_position_ic(
+            ppm=ppm, background=self.bg_freq, pseudocount=0.001)
+        return per_pos_ic
+
+
 class ExpandSeqletsToFillPattern(AbstractAggSeqletPostprocessor):
 
     def __init__(self, track_set, flank_to_add=0,
@@ -263,7 +276,7 @@ class ReassignSeqletsFromSmallClusters(AbstractAggSeqletPostprocessor):
                 large_patterns, new_assignments =\
                     self.seqlet_assigner(patterns=large_patterns,
                                          seqlets_to_assign=seqlets_to_assign,
-                                         merge_into_existing_patterns=True)
+                                         merge_into_existing_patterns=False)
             large_patterns = self.postprocessor(large_patterns)
             return large_patterns
         else:
@@ -386,13 +399,16 @@ class AssignSeqletsByBestMetric(object):
 
         if (merge_into_existing_patterns):
             new_patterns = patterns
-            for pattern,x in zip(patterns, seqlet_and_alnmnt_grps):
-                pattern.merge_seqlets_and_alnmts(
-                    seqlets_and_alnmts=x,
-                    aligner=self.pattern_aligner) 
         else:
-            new_patterns = [core.AggregatedSeqlet(seqlets_and_alnmts_arr=x)
-                for x in seqlet_and_alnmnt_grps if len(x) > 0]
+            #Make a copy of each one
+            new_patterns = [
+                core.AggregatedSeqlet(pattern._seqlets_and_alnmts.copy())
+                for pattern in patterns]
+            
+        for pattern,x in zip(new_patterns, seqlet_and_alnmnt_grps):
+            pattern.merge_seqlets_and_alnmts(
+                seqlets_and_alnmts=x,
+                aligner=self.pattern_aligner) 
 
         return new_patterns, seqlet_assignments
 
