@@ -236,6 +236,11 @@ class TfModiscoSeqletsToPatternsFactory(object):
                        track_signs,
                        other_comparison_track_names=[]):
 
+        bg_freq = np.mean(
+            track_set.track_name_to_data_track[onehot_track_name].fwd_tracks,
+            axis=(0,1))
+        assert len(bg_freq.shape)==1
+
         assert len(track_signs)==len(hypothetical_contribs_track_names)
         assert len(track_signs)==len(contrib_scores_track_names)
 
@@ -271,8 +276,7 @@ class TfModiscoSeqletsToPatternsFactory(object):
                 seqlets_to_1d_embedder=seqlets_to_1d_embedder,
                 affinity_mat_from_1d=\
                     affinitymat.core.NumpyCosineSimilarity(
-                        verbose=self.verbose,
-                        gpu_batch_size=None),
+                        verbose=self.verbose),
                 verbose=self.verbose)
 
         affmat_from_seqlets_with_nn_pairs =\
@@ -347,9 +351,10 @@ class TfModiscoSeqletsToPatternsFactory(object):
             aggregator.ExpandSeqletsToFillPattern(
                 track_set=track_set,
                 flank_to_add=self.initial_flank_to_add).chain(
-            aggregator.TrimToBestWindow(
+            aggregator.TrimToBestWindowByIC(
                 window_size=self.trim_to_window_size,
-                track_names=contrib_scores_track_names)).chain(
+                onehot_track_name=onehot_track_name,
+                bg_freq=bg_freq)).chain(
             aggregator.ExpandSeqletsToFillPattern(
                 track_set=track_set,
                 flank_to_add=self.initial_flank_to_add))
@@ -502,11 +507,14 @@ class SeqletsToPatternsResults(object):
 
     def __init__(self,
                  each_round_initcluster_motifs, 
-                 patterns, cluster_results,
+                 patterns, 
+                 patterns_withoutreassignment,
+                 cluster_results,
                  total_time_taken, success=True, **kwargs):
         self.each_round_initcluster_motifs = each_round_initcluster_motifs
         self.success = success
         self.patterns = patterns
+        self.patterns_withoutreassignment = patterns_withoutreassignment
         self.cluster_results = cluster_results
         self.total_time_taken = total_time_taken
         self.__dict__.update(**kwargs)
@@ -548,16 +556,25 @@ class SeqletsToPatternsResults(object):
                         track_set=track_set)
             patterns = util.load_patterns(grp=grp["patterns"],
                                           track_set=track_set) 
+            if "patterns_withoutreassignment" in grp:
+                patterns_withoutreassignment = util.load_patterns(
+                    grp=grp["patterns_withoutreassignment"],
+                    track_set=track_set) 
+            else: #backwards compatibility
+                patterns_withoutreassignment = []
             cluster_results = None
             total_time_taken = None
             return cls(
                 each_round_initcluster_motifs=each_round_initcluster_motifs,
                 patterns=patterns,
+                patterns_withoutreassignment=patterns_withoutreassignment,
                 cluster_results=cluster_results,
                 total_time_taken=total_time_taken)
         else:
             return cls(success=False, patterns=None, cluster_results=None,
-                       total_time_taken=None)
+                       total_time_taken=None,
+                       each_round_initcluster_motifs=None,
+                       patterns_withoutreassignment=None)
 
     def save_hdf5(self, grp):
         grp.attrs["success"] = self.success
@@ -567,6 +584,9 @@ class SeqletsToPatternsResults(object):
                     grp=grp.create_group("each_round_initcluster_motifs"))
             util.save_patterns(self.patterns,
                                grp.create_group("patterns"))
+            util.save_patterns(
+                self.patterns_withoutreassignment,
+                grp.create_group("patterns_withoutreassignment"))
             self.cluster_results.save_hdf5(grp.create_group("cluster_results"))   
             grp.attrs['total_time_taken'] = self.total_time_taken
 
@@ -651,7 +671,6 @@ class TfModiscoSeqletsToPatterns(AbstractSeqletsToPatterns):
                                   " with "+str(motif.num_seqlets)
                                   +" seqlets due to sign disagreement")
                         cluster_to_eliminated_motif[i] = motif
-                cluster_to_motif[i] = motif
         return cluster_to_motif, cluster_to_eliminated_motif
 
 
@@ -706,6 +725,8 @@ class TfModiscoSeqletsToPatterns(AbstractSeqletsToPatterns):
                         patterns=None,
                         seqlets=None,
                         affmat=None,
+                        each_round_initcluster_motifs=None,
+                        patterns_withoutreassignment=None,
                         cluster_results=None, 
                         total_time_taken=None,
                         success=False)
@@ -843,9 +864,9 @@ class TfModiscoSeqletsToPatterns(AbstractSeqletsToPatterns):
                     min_seqlets_in_motif=0)
 
             #obtain unique seqlets from adjusted motifs
-            seqlets = dict([(y.exidx_start_end_string, y)
+            seqlets = list(dict([(y.exidx_start_end_string, y)
                              for x in cluster_to_motif.values()
-                             for y in x.seqlets]).values()
+                             for y in x.seqlets]).values())
 
         if (self.verbose):
             print("Got "+str(len(cluster_to_motif.values()))+" clusters")
@@ -887,6 +908,8 @@ class TfModiscoSeqletsToPatterns(AbstractSeqletsToPatterns):
             sys.stdout.flush()
         reassigned_patterns = self.seqlet_reassigner(merged_patterns)
         final_patterns = self.final_postprocessor(reassigned_patterns)
+        final_patterns_withoutreassignment =\
+            self.final_postprocessor(merged_patterns)
         if (self.verbose):
             print("Got "+str(len(final_patterns))
                   +" patterns after reassignment")
@@ -903,6 +926,7 @@ class TfModiscoSeqletsToPatterns(AbstractSeqletsToPatterns):
         results = SeqletsToPatternsResults(
             each_round_initcluster_motifs=each_round_initcluster_motifs,             
             patterns=final_patterns,
+            patterns_withoutreassignment=final_patterns_withoutreassignment,
             seqlets=filtered_seqlets, #last stage of filtered seqlets
             #affmat=filtered_affmat,
             cluster_results=cluster_results, 
