@@ -117,6 +117,7 @@ class SubMetaclusterResults(object):
 
 def prep_track_set(task_names, contrib_scores,
                     hypothetical_contribs, one_hot,
+                    custom_perpos_contribs=None,
                     revcomp=True, other_tracks=[]):
     contrib_scores_tracks = [
         core.DataTrack(
@@ -141,9 +142,22 @@ def prep_track_set(task_names, contrib_scores,
                         rev_tracks=([x[::-1, ::-1] for x in one_hot]
                                     if revcomp else None),
                         has_pos_axis=True)
+
+    custom_perpos_contrib_tracks = []
+    if (custom_perpos_contribs):
+        custom_perpos_contrib_tracks = [
+            core.DataTrack(name=key+"_custom_perposcontribs",
+                           fwd_tracks=custom_perpos_contribs[key],
+                           rev_tracks=([x[::-1] for x in
+                                       custom_perpos_contribs[key]]
+                                       if revcomp else None),
+                           has_pos_axis=True)
+                           for key in task_names] 
+
     track_set = core.TrackSet(
                     data_tracks=contrib_scores_tracks
-                    +hypothetical_contribs_tracks+[onehot_track]+other_tracks)
+                    +hypothetical_contribs_tracks+[onehot_track]
+                    +custom_perpos_contrib_tracks+other_tracks)
     return track_set
 
 
@@ -153,7 +167,6 @@ class TfModiscoWorkflow(object):
                  seqlets_to_patterns_factory=
                  seqlets_to_patterns.TfModiscoSeqletsToPatternsFactory(),
                  sliding_window_size=21, flank_size=10,
-                 histogram_bins=100, percentiles_in_bandwidth=10, 
                  overlap_portion=0.5,
                  min_metacluster_size=100,
                  min_metacluster_size_frac=0.01,
@@ -174,8 +187,6 @@ class TfModiscoWorkflow(object):
         self.seqlets_to_patterns_factory = seqlets_to_patterns_factory
         self.sliding_window_size = sliding_window_size
         self.flank_size = flank_size
-        self.histogram_bins = histogram_bins
-        self.percentiles_in_bandwidth = percentiles_in_bandwidth
         self.overlap_portion = overlap_portion
         self.min_metacluster_size = min_metacluster_size
         self.min_metacluster_size_frac = min_metacluster_size_frac
@@ -211,7 +222,6 @@ class TfModiscoWorkflow(object):
                        plot_save_dir="figures"):
 
         print_memory_use()
-
         self.coord_producer = coordproducers.FixedWindowAroundChunks(
             sliding=self.sliding_window_size,
             flank=self.flank_size,
@@ -225,18 +235,21 @@ class TfModiscoWorkflow(object):
             verbose=self.verbose,
             plot_save_dir=plot_save_dir) 
 
+        custom_perpos_contribs = per_position_contrib_scores
+        if (per_position_contrib_scores is None):
+            per_position_contrib_scores = OrderedDict([
+                (x, [np.sum(s,axis=1) for s in contrib_scores[x]])
+                for x in task_names])
+
         track_set = prep_track_set(
                         task_names=task_names,
                         contrib_scores=contrib_scores,
                         hypothetical_contribs=hypothetical_contribs,
                         one_hot=one_hot,
                         revcomp=revcomp,
+                        custom_perpos_contribs=custom_perpos_contribs,
                         other_tracks=other_tracks)
 
-        if (per_position_contrib_scores is None):
-            per_position_contrib_scores = OrderedDict([
-                (x, [np.sum(s,axis=1) for s in contrib_scores[x]])
-                for x in task_names])
 
         multitask_seqlet_creation_results = core.MultiTaskSeqletCreator(
             coord_producer=self.coord_producer,
@@ -284,7 +297,9 @@ class TfModiscoWorkflow(object):
         task_name_to_value_provider = OrderedDict([
             (task_name,
              value_provider.TransformCentralWindowValueProvider(
-                track_name=task_name+"_contrib_scores",
+                track_name=(task_name+"_contrib_scores"
+                            if custom_perpos_contribs is None else
+                            task_name+"_custom_perposcontribs"),
                 central_window=self.sliding_window_size,
                 val_transformer= 
                  coord_producer_results.tnt_results.val_transformer))
